@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2023 Open Text.
+# Copyright (c) 2018-2024 Open Text.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -109,10 +109,12 @@ class AuthenticationTestCase(VerticaPythonIntegrationTestCase):
                 cur.execute("DROP AUTHENTICATION IF EXISTS testIPv6hostHash CASCADE")
                 cur.execute("DROP AUTHENTICATION IF EXISTS testlocalHash CASCADE")
 
-    def test_oauth(self):
+    def test_oauth_access_token(self):
         self.require_protocol_at_least(3 << 16 | 11)
         if not self._oauth_info['access_token']:
-            self.skipTest('OAuth not set')
+            self.skipTest('OAuth access token not set')
+        if not self._oauth_info['user'] and not self._conn_info['database']:
+            self.skipTest('Both database and oauth_user are not set')
 
         self._conn_info['user'] = self._oauth_info['user']
         self._conn_info['oauth_access_token'] = self._oauth_info['access_token']
