@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2023 Open Text.
+# Copyright (c) 2018-2024 Open Text.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -42,7 +42,9 @@ class ParseDSNTestCase(VerticaPythonUnitTestCase):
                'session_label=vpclient&unicode_error=strict&'
                'log_path=/home/admin/vClient.log&log_level=DEBUG&'
                'oauth_access_token=GciOiJSUzI1NiI&'
-               'workload=python_test_workload&'
+               'workload=python_test_workload&tlsmode=verify-ca&'
+               'tls_cafile=tls/ca_cert.pem&tls_certfile=tls/cert.pem&'
+               'tls_keyfile=tls/key.pem&'
                'kerberos_service_name=krb_service&kerberos_host_name=krb_host')
         expected = {'database': 'db1', 'host': 'localhost', 'user': 'john',
                     'password': 'pwd', 'port': 5433, 'log_level': 'DEBUG',
@@ -50,6 +52,10 @@ class ParseDSNTestCase(VerticaPythonUnitTestCase):
                     'log_path': '/home/admin/vClient.log',
                     'oauth_access_token': 'GciOiJSUzI1NiI',
                     'workload': 'python_test_workload',
+                    'tlsmode': 'verify-ca',
+                    'tls_cafile': 'tls/ca_cert.pem',
+                    'tls_certfile': 'tls/cert.pem',
+                    'tls_keyfile': 'tls/key.pem',
                     'kerberos_service_name': 'krb_service',
                     'kerberos_host_name': 'krb_host'}
         parsed = parse_dsn(dsn)
